<?php
/*
Plugin Name: 1WD Series
Plugin URI: http://www.1stwebdesigner.com/
Description: Creates "Series" custom post type, and makes possible relate normal posts inside one series
Author: Rochester Oliveira
Version: 1.0
Author URI: http://www.1stwebdesigner.com/author/rochester/
*/
// our prefix will be wd_ to prevent conflict with other functions
	//setting up our post type
	function wd_registerPostType() {
		//for better understanding when create / edit series we change some default labels
		$labels = array(
			'name' => _x('Series', 'wd_series'),
			'singular_name' => _x('Series', 'wd_series'),
			'add_new' => _x('New Series', 'wd_series'),
			'add_new_item' => __('Add New Series'),
			'edit_item' => __('Edit Series'),
			'new_item' => __('New Series'),
			'view_item' => __('View Series'),
			'search_items' => __('Search Series')
		);
		$args = array(
			'labels' => $labels,
			'public' => true,
			'publicly_queryable' => true,
			'show_ui' => true, 
			'query_var' => true,
			'rewrite' => array( 'slug' => 'series', 'with_front' => false ),
			'capability_type' => 'post',
			'menu_position' => null,
			'supports' => array('title','editor')
		); 
		register_post_type('wd_series',$args);
	}
	
	//creates meta boxes, for corectly input series data in 'wd_series' and 'post'
	function wd_call_meta() {
		add_meta_box(
			'series-data', // id of the <div> we'll add
			'Series Custom Data', //title
			'wd_series_options', // callback function that will echo the box content
			'wd_series', // where to add the box: on "post", "page", or "link" page 
			'side', //positioning
			'low' //positioning
		);
		add_meta_box(
			'series-data',
			'Series setting',
			'wd_post_options',
			'post',
			'side',
			'low'
		);
	}
	
	//custom post fields for series
	function wd_series_options() {
		global $post;
		//get saved data
		$custom    = get_post_custom($post->ID);
		$open      = $custom["open"][0];
		$numFields = $custom["size"][0];
		
		$openNo = $openYes = "";
		if ($open == "yes") {
			$openYes = "checked = 'checked'";
		} else {
			$openNo = "checked = 'checked'";
		}
		
		$series_items = array();
		$i = 0;
		while($i < $numFields) {
			$i++;
			$key = "name_".$i;
			$series_items[$i] = $custom[$key][0];
		}
		$numFields++;
?>
	<table>
		<tr class="space">
			<td><label>Open?</label></td>
			<td>
				<label><input id="wd_open" name="wd_open" type="radio" value="yes" <?php echo $openYes; ?> /> Yes, it is open.</label><br />
				<label><input id="wd_open" name="wd_open" type="radio" value="no" <?php echo $openNo; ?> /> No, it is closed</label>
			</td>
		</tr>
	</table>
	<table class="default-line">
		<tr>
			<td><input id="order_K" name="order_K" type="text" value="" size="5" /></td>
			<td><input id="name_K" name="name_K" type="text" value="" /></td>
			<td class="check"><input id="remove_K" name="remove_K" type="checkbox" value="remove" /></td>
		</tr>
	</table>
	<input type="hidden" id="NumFields" name="NumFields" value="<?php echo $numFields; ?>" />
	<table>
		<tr>
			<td>Order</td>
			<td>PostName</td>
			<td class="checkboxTd">Remove?</td>
		</tr>
	</table>
	<div class="ins">
	<?php
		foreach($series_items as $key => $name) {
			echo "<table>";
				echo "<tr>";
					echo "<td><input id='order_$key' name='order_$key' type='text' value='$key' size='5' /></td>";
					echo "<td><input id='name_$key' name='name_$key' type='text' value='$name' /></td>";
					echo "<td class='check'><input id='remove_$key' name='remove_$key' type='checkbox' value='remove' /></td>";
				echo "</tr>";
			echo "</table>";
		}
	?>
	</div>
	<table>
		<tr id="addItem">
			<td colspan="3"><a onclick="addLine();" class="link" id="clickLink">Add new line</a></td>
		</tr>
	</table>
<style type="text/css">
	.default-line {
		display: none;
	}
	#series-data table {
		width: 100%
	}
		#series-data tr.space td {
			padding-bottom: 10px;
		}
			#series-data .space label {
				padding-right: 20px
			}
		#series-data .check {
			text-align: center;
		}
		#addItem td {
			padding-top: 10px;
			text-align: right;
		}
	.checkboxTd {
		width: 20px;
	}
	.link { cursor: pointer; }
</style>
<script type="text/javascript">
	function addLine() {		
		jQuery(".default-line").clone().appendTo(".ins");
		var numElem = jQuery("#NumFields").attr('value');
		jQuery(".ins .default-line").fadeIn().removeClass('default-line');
		correctsLine(numElem);
	}
	function correctsLine(nID) {
		//goes switching K to correct number of line
		fieldName  = "order_" + nID;
		jQuery(".ins input#order_K").attr('value', ( parseInt(nID)+1) ).attr('id', fieldName).attr('name', fieldName);
		
		fieldName  = "name_" + nID;
		jQuery(".ins input#name_K").attr('id', fieldName).attr('name', fieldName);
		
		fieldName  = "remove_" + nID;
		jQuery(".ins input#remove_K").attr('id', fieldName).attr('name', fieldName);
		
		//corrects number of fields
		nID++;
		jQuery("#NumFields").attr('value', nID);
	}
	jQuery(window).load(function(){
		addLine(); //adds first (blank) row
	});
</script>
<?php
	} //end series meta
	
	//posts meta
	function wd_post_options() {
		global $post;
		
		$custom = get_post_custom($post->ID);
		$exist_series = $custom["series_id"][0];
		$order_series = $custom["series_order"][0];
		$closing = $name = "";
		if(!empty($exist_series)) {
			$open = get_post_meta($exist_series, "open", true);
			if ($open == "no") {
				$size = get_post_meta($exist_series, "size", true);
				if ($size == $order_series) {
					$closing = "checked = 'checked'";
				}
				$key = "name_".$order_series;
				$name = get_post_meta($exist_series, $key, true);
			}
		}
?>
	<table>
		<tr>
			<td><label>Add to Series (leave blank to remove)</label></td>
			<td>
				<label>
					<select name="wd_exist_series" id="wd_exist_series">
						<option>--Select existing series</option>
						<?php
							$series_print = 0; //when this post is assigned to a closed series, we have to show it :D
							$args = array( 'post_type' => 'wd_series', 'posts_per_page' => -1, 'meta_key' => 'open', 'meta_value' => 'yes'  ) ;
							$loop = get_posts( $args );
							foreach( $loop as $series ) :
								if ($series->ID == $exist_series) {
									$selected = "selected = 'selected'";
									$series_print = 1;
								} else {
									$selected = "";
								}
								echo "<option value='".$series->ID."' $selected>".get_the_title( $series->ID )."</option>";
							endforeach;
							
							if (empty($series_print)) {
								$data = get_post($exist_series);
								echo "<option value='".$data->ID."' selected = 'selected'>".get_the_title($exist_series)."</option>";
							}
						?>
					</select>
				</label><br />
				<label><input id="wd_new_series" name="wd_new_series" type="text" value="New Series Name" /></label>
			</td>
		</tr>
		<tr>
			<td>Order in series (leave blank to be last)</td>
			<td>
				<label><input id="wd_order" name="wd_order" type="text" value="<?php echo $order_series; ?>" /></label>
			</td>
		</tr>
		<tr>
			<td>Close series?</td>
			<td>
				<label><input id="wd_end" name="wd_end" type="checkbox" value="Yes" <?php echo $closing; ?> /> Yes</label>
			</td>
		</tr>
		<tr class="nospace">
			<td colspan="2"><label for="wd_name">Display name in series</label></td>
		</tr>
		<tr>
			<td colspan="2"><input id="wd_name" name="wd_name" type="text" value="<?php echo $name; ?>" /></td>
		</tr>
	</table>
<style type="text/css">
	#series-data table {
		width: 100%
	}
		#series-data tr td {
			padding-bottom: 10px;
		}
		#series-data .nospace td {
			padding-bottom: 0
		}
</style>
<?php
	} //end post meta
	//saves our custom meta data
	function wd_meta($post_id, $post = null) {
		//gets our POST custom data and saves it as meta keys, when needed
		/*
		CPT in posts: series_id
		*/
		if ($post->post_type == "post") {
			$title = get_the_title($post_id);
			
			$old_series = get_post_meta($post_id, "series_id", true);
			//compare old series data, and see if it needs to be rewritten
			$exist_series = (int) @$_POST["wd_exist_series"];
			$new_series   = @$_POST["wd_new_series"];
			$order        = @$_POST["wd_order"];
			$name         = @$_POST["wd_name"];
			$closing      = @$_POST["wd_end"];
			
			if(!empty($new_series) && $new_series != "New Series Name") {
				//our series have changed, let's delete this post from old series and "resave" it!
				$key = "order_".$post_id;
				$old_order =  get_post_meta($old_series, $key, true);
				delete_post_meta($old_series, $key);
				
				$key = "post_".$old_order;
				delete_post_meta($old_series, $key);
				
				$key = "name_".$old_order;
				delete_post_meta($old_series, $key);
				
				//update old series size
				$old_size = get_post_meta($old_series, "size", true);
				$old_size--;
				update_post_meta($old_series, "size", $old_size);
				
				//we have a new series, so let's create it, dude!
				$args = array(
							 'post_title'  => $new_series,
							 'post_status' => 'publish',
							 'post_type'   => 'wd_series'
						  );
				$series_id = wp_insert_post( $args );
									
				if (empty($name)) {
					//we don't have any name, so let's use post name
					$name = $title;
				}
				//add this post to series_id and its wd_order and wd_name
				update_post_meta($series_id, "size", 1);
				update_post_meta($series_id, "post_1", $post_id);
				update_post_meta($series_id, "name_1", $name);
				$key = "order_".$post_id;
				update_post_meta($series_id, $key, 1);
				
				update_post_meta( $post_id, "series_id", $series_id );
			}
			elseif (!empty($exist_series)) {
				// we have a series, let's see if it is still the same
				if ( $exist_series != $old_series) {
					//our series have changed, let's delete this post from old series and "resave" it!
					$key = "order_".$post_id;
					$old_order =  get_post_meta($old_series, $key, true);
					delete_post_meta($old_series, $key);
				
					$key = "post_".$old_order;
					delete_post_meta($old_series, $key);
					
					$key = "name_".$old_order;
					delete_post_meta($old_series, $key);
					
					//update old series size
					$old_size = get_post_meta($old_series, "size", true);
					$old_size--;
					update_post_meta($old_series, "size", $old_size);
					
					//update series size
					$size = get_post_meta($exist_series, "size", true);
					$size++;
					update_post_meta($exist_series, "size", $size);
					
					if (!empty($new_order)) {
						//we do have a new order to set, so let's follow it
						$order = $new_order;
					} else {
						//we don't have any order to set, so let's check it as last item
						$order = $size;
					}
					$key = "post_".$order;
					update_post_meta( $exist_series, $key, $post_id );
					
					if (empty($name)) {
						//we don't have any name, so let's use post name
						$name = $title;
					}
					$key = "name_".$order;
					update_post_meta( $exist_series, $key, $name );
					
					$key = "order_".$post_id;
					update_post_meta( $exist_series, $key, $order );
						
					//save new series on this post
					update_post_meta( $post_id, "series_id", $exist_series );
				}
			}
			//closing if needed
			if (!empty($closing)) {
				update_post_meta( $series_id, "open", "no" );
			} else {
				update_post_meta( $series_id, "open", "yes" );
			}
		}
		/*
		CPT in series: open, size, post_n, name_n
		*/
		elseif( $post->post_type == "wd_series"  ) {
			//update series state (open / closed)
			$open = @$_POST["wd_open"];
			update_post_meta( $post_id, "open", $open );
			
			$size = @$_POST["NumFields"];
			$i = 0;
			
			$organize = array();
			while ($i < $size) {
				//let's pre-organize all posts
				$survive = $key = $remove = $order = $name = $post = null;
				
				$key = "remove_".$i;
				$remove = @$_POST[$key];
				
				if (empty($remove)) {
					//we won't delete this guy, and we'll put he in his right order
					$key = "order_".$i;
					$order = @$_POST[$key];
					
					$key = "name_".$i;
					$name = @$_POST[$key];
					
					$key = "post_".$i;
					$post = get_post_meta($post_id, $key, true);
					
					if (!empty($name)) {
						$organize[$order] = array( 'name' => $name, 'post' => $post);
						$survive = true;
					}
				}
				//we will pre delete everybody, to prevent trash in here
				$key = "name_".$i;
				delete_post_meta($post_id, $key);
				
				$key = "post_".$i;
				$post = get_post_meta($post_id, $key, true);
				delete_post_meta($post_id, $key);
				
				$key = "order_".$post;
				delete_post_meta($post_id, $key);
				
				//if it won't survive, we delete series_id from post_id
				if(!$survive) {					
					$key = "series_id";
					delete_post_meta($post, $key);
				}
				$i++;
			}
			//let's correctly order this
			ksort($organize);
			$i = 0;
			foreach($organize as $item) {
				$i++;
				
				$key = "name";
				$nam = $item[$key];
				$key = "name_".$i;
				update_post_meta( $post_id, $key, $nam );
				
				$key  = "post";
				$post = $item[$key];
				if(!empty($post)) {
					$key = "post_".$i;
					update_post_meta( $post_id, $key, $post );
					$key = "order_".$post;
					update_post_meta( $post_id, $key, $i );
				}
			}
			
			$size = count($organize);			
			update_post_meta( $post_id, "size", $size );
		}
	}
	//add custom meta when we save posts
	add_action("wp_insert_post", 'wd_meta', 10, 2);
	
	if (is_admin()) {
		//register metaboxes
		add_action('admin_menu', 'wd_call_meta');
	}
	
	add_action("init", "wd_registerPostType");
	
// Registering our functions to display the series content
//display item with or without link
function wd_item( $name, $postItem ) {
	if ( ! empty ( $postItem ) ) {
		$link = get_permalink($postItem);
		echo "<a href='$link' title='$name'>$name</a>";
	} else {
		echo $name;
	}
}
//output!
function wd_series ($series) {
	$title = get_the_title($series);
	$meta  = get_post_custom($series);
	
	echo "<h3>$title</h3>";
	$size = $meta["size"][0];
	$i = 1;
	echo "<ol>";
	while ($i <= $size) {
		$key = "name_".$i;
		$name = $meta[$key][0];
		
		$key = "post_".$i;
		$post_item = $meta[$key][0];
		
		echo "<li>";
			wd_item( $name, $post_item);
		echo "</li>";
		
		$i++;
		
		$name = $post_item = null;
	}
	echo "</ol>";
}
// [wd-series]
function wd_series_shortcode() {
	global $post;
	
	$series = get_post_meta($post->ID, "series_id", true);
	
	wd_series($series);
}
add_shortcode( 'wd-series', 'wd_series_shortcode' );

?>